@@ -1,68 +1,88 @@
-//! Core of SQLx, the rust SQL toolkit. Not intended to be used directly.
-
+//! Core of SQLx, the rust SQL toolkit.
+//! Not intended to be used directly.
+#![recursion_limit = "512"]
+#![warn(future_incompatible, rust_2018_idioms)]
+#![allow(clippy::needless_doctest_main, clippy::type_complexity)]
+//
+// Allows an API be documented as only available in some specific platforms.
+// <https://doc.rust-lang.org/unstable-book/language-features/doc-cfg.html>
+#![cfg_attr(docsrs, feature(doc_cfg))]
+//
 // When compiling with support for SQLite we must allow some unsafe code in order to
 // interface with the inherently unsafe C module. This unsafe code is contained
 // to the sqlite module.
 #![cfg_attr(feature = "sqlite", deny(unsafe_code))]
 #![cfg_attr(not(feature = "sqlite"), forbid(unsafe_code))]
-#![recursion_limit = "512"]
-#![cfg_attr(docsrs, feature(doc_cfg))]
-#![cfg_attr(all(test, feature = "bench"), feature(test))]
-// #![warn(missing_docs)]
 
-#[cfg(all(test, feature = "bench"))]
-extern crate test;
-
-// HACK: Allow a feature name the same name as a dependency
 #[cfg(feature = "bigdecimal")]
 extern crate bigdecimal_ as bigdecimal;
 
-mod runtime;
+#[macro_use]
+mod ext;
 
 #[macro_use]
 pub mod error;
 
-#[cfg(any(feature = "mysql", feature = "postgres"))]
-#[macro_use]
-mod io;
-
-pub mod connection;
-pub mod cursor;
-pub mod database;
-pub mod value;
-
-#[macro_use]
-pub mod executor;
-
-pub mod transaction;
-mod url;
-
 #[macro_use]
 pub mod arguments;
-pub mod decode;
-
-#[doc(hidden)]
-pub mod describe;
-
-pub mod encode;
-pub mod pool;
-pub mod query;
 
 #[macro_use]
-pub mod query_as;
+pub mod pool;
 
+pub mod connection;
+
+#[macro_use]
+pub mod transaction;
+
+#[macro_use]
+pub mod encode;
+
+#[macro_use]
+pub mod decode;
+
+#[macro_use]
 pub mod types;
 
 #[macro_use]
-pub mod row;
+pub mod query;
 
-#[cfg(any(feature = "mysql", feature = "postgres", feature = "sqlite"))]
 #[macro_use]
-mod logging;
+pub mod acquire;
 
-#[cfg(feature = "mysql")]
-#[cfg_attr(docsrs, doc(cfg(feature = "mysql")))]
-pub mod mysql;
+#[macro_use]
+pub mod column;
+
+#[macro_use]
+pub mod statement;
+
+mod common;
+pub mod database;
+pub mod describe;
+pub mod done;
+pub mod executor;
+pub mod from_row;
+mod io;
+mod logger;
+mod net;
+pub mod query_as;
+pub mod query_scalar;
+pub mod row;
+pub mod type_info;
+pub mod value;
+
+#[cfg(feature = "migrate")]
+pub mod migrate;
+
+#[cfg(all(
+    any(
+        feature = "postgres",
+        feature = "mysql",
+        feature = "mssql",
+        feature = "sqlite"
+    ),
+    feature = "any"
+))]
+pub mod any;
 
 #[cfg(feature = "postgres")]
 #[cfg_attr(docsrs, doc(cfg(feature = "postgres")))]
@@ -72,4 +92,14 @@ pub mod postgres;
 #[cfg_attr(docsrs, doc(cfg(feature = "sqlite")))]
 pub mod sqlite;
 
-pub use error::{Error, Result};
+#[cfg(feature = "mysql")]
+#[cfg_attr(docsrs, doc(cfg(feature = "mysql")))]
+pub mod mysql;
+
+#[cfg(feature = "mssql")]
+#[cfg_attr(docsrs, doc(cfg(feature = "mssql")))]
+pub mod mssql;
+
+/// sqlx uses ahash for increased performance, at the cost of reduced DoS resistance.
+use ahash::AHashMap as HashMap;
+//type HashMap<K, V> = std::collections::HashMap<K, V, ahash::RandomState>;
