@@ -5,6 +5,7 @@
 //! | Rust type                             | Postgres type(s)                                     |
 //! |---------------------------------------|------------------------------------------------------|
 //! | `bool`                                | BOOL                                                 |
+//! | `i8`                                  | "CHAR"                                               |
 //! | `i16`                                 | SMALLINT, SMALLSERIAL, INT2                          |
 //! | `i32`                                 | INT, SERIAL, INT4                                    |
 //! | `i64`                                 | BIGINT, BIGSERIAL, INT8                              |
@@ -12,6 +13,27 @@
 //! | `f64`                                 | DOUBLE PRECISION, FLOAT8                             |
 //! | `&str`, `String`                      | VARCHAR, CHAR(N), TEXT, NAME                         |
 //! | `&[u8]`, `Vec<u8>`                    | BYTEA                                                |
+//! | [`PgInterval`]                        | INTERVAL                                             |
+//! | [`PgRange<T>`]                        | INT8RANGE, INT4RANGE, TSRANGE, TSTZTRANGE, DATERANGE, NUMRANGE |
+//! | [`PgMoney`]                           | MONEY                                                |
+//!
+//! [`PgInterval`]: struct.PgInterval.html
+//! [`PgRange<T>`]: struct.PgRange.html
+//! [`PgMoney`]: struct.PgMoney.html
+//!
+//! ### [`bigdecimal`](https://crates.io/crates/bigdecimal)
+//! Requires the `bigdecimal` Cargo feature flag.
+//!
+//! | Rust type                             | Postgres type(s)                                        |
+//! |---------------------------------------|------------------------------------------------------|
+//! | `bigdecimal::BigDecimal`              | NUMERIC                                              |
+//!
+//! ### [`decimal`](https://crates.io/crates/rust_decimal)
+//! Requires the `decimal` Cargo feature flag.
+//!
+//! | Rust type                             | Postgres type(s)                                        |
+//! |---------------------------------------|------------------------------------------------------|
+//! | `rust_decimal::Decimal`               | NUMERIC                                              |
 //!
 //! ### [`chrono`](https://crates.io/crates/chrono)
 //!
@@ -24,6 +46,7 @@
 //! | `chrono::NaiveDateTime`               | TIMESTAMP                                            |
 //! | `chrono::NaiveDate`                   | DATE                                                 |
 //! | `chrono::NaiveTime`                   | TIME                                                 |
+//! | [`PgTimeTz`]                          | TIMETZ                                               |
 //!
 //! ### [`time`](https://crates.io/crates/time)
 //!
@@ -35,6 +58,9 @@
 //! | `time::OffsetDateTime`                | TIMESTAMPTZ                                          |
 //! | `time::Date`                          | DATE                                                 |
 //! | `time::Time`                          | TIME                                                 |
+//! | [`PgTimeTz`]                          | TIMETZ                                               |
+//!
+//! [`PgTimeTz`]: struct.PgTimeTz.html
 //!
 //! ### [`uuid`](https://crates.io/crates/uuid)
 //!
@@ -52,6 +78,14 @@
 //! |---------------------------------------|------------------------------------------------------|
 //! | `ipnetwork::IpNetwork`                | INET, CIDR                                           |
 //!
+//! ### [`bit-vec`](https://crates.io/crates/bit-vec)
+//!
+//! Requires the `bit-vec` Cargo feature flag.
+//!
+//! | Rust type                             | Postgres type(s)                                     |
+//! |---------------------------------------|------------------------------------------------------|
+//! | `bit_vec::BitVec`                     | BIT, VARBIT                                          |
+//!
 //! ### [`json`](https://crates.io/crates/serde_json)
 //!
 //! Requires the `json` Cargo feature flag.
@@ -122,31 +156,36 @@
 //! enum Mood { Sad = 0, Ok = 1, Happy = 2 }
 //! ```
 //!
-//! # Nullable
-//!
-//! In addition, `Option<T>` is supported where `T` implements `Type`. An `Option<T>` represents
-//! a potentially `NULL` value from Postgres.
-//!
 
-use crate::decode::Decode;
-use crate::postgres::protocol::TypeId;
-use crate::postgres::{PgValue, Postgres};
+use crate::postgres::type_info::PgTypeKind;
+use crate::postgres::{PgTypeInfo, Postgres};
+use crate::types::Type;
 
 mod array;
 mod bool;
 mod bytes;
 mod float;
 mod int;
+mod interval;
+mod money;
+mod range;
 mod record;
 mod str;
+mod tuple;
+mod void;
 
-// internal types used by other types to encode or decode related formats
-#[doc(hidden)]
-pub mod raw;
+#[cfg(any(feature = "chrono", feature = "time"))]
+mod time_tz;
 
 #[cfg(feature = "bigdecimal")]
 mod bigdecimal;
 
+#[cfg(any(feature = "bigdecimal", feature = "decimal"))]
+mod numeric;
+
+#[cfg(feature = "decimal")]
+mod decimal;
+
 #[cfg(feature = "chrono")]
 mod chrono;
 
@@ -162,98 +201,28 @@ mod json;
 #[cfg(feature = "ipnetwork")]
 mod ipnetwork;
 
-// Implement `Decode` for all postgres types
-// The concept of a nullable `RawValue` is db-specific
-// `Type` is implemented generically at src/types.rs
-impl<'de, T> Decode<'de, Postgres> for Option<T>
-where
-    T: Decode<'de, Postgres>,
-{
-    fn decode(value: PgValue<'de>) -> crate::Result<Self> {
-        Ok(if value.get().is_some() {
-            Some(<T as Decode<Postgres>>::decode(value)?)
-        } else {
-            None
-        })
+#[cfg(feature = "bit-vec")]
+mod bit_vec;
+
+pub use interval::PgInterval;
+pub use money::PgMoney;
+pub use range::PgRange;
+
+#[cfg(any(feature = "chrono", feature = "time"))]
+pub use time_tz::PgTimeTz;
+
+// used in derive(Type) for `struct`
+// but the interface is not considered part of the public API
+#[doc(hidden)]
+pub use record::{PgRecordDecoder, PgRecordEncoder};
+
+// Type::compatible impl appropriate for arrays
+fn array_compatible<E: Type<Postgres>>(ty: &PgTypeInfo) -> bool {
+    // we require the declared type to be an _array_ with an
+    // element type that is acceptable
+    if let PgTypeKind::Array(element) = &ty.kind() {
+        return E::compatible(&element);
     }
-}
 
-// Try to resolve a _static_ type name from an OID
-pub(crate) fn try_resolve_type_name(oid: u32) -> Option<&'static str> {
-    Some(match TypeId(oid) {
-        TypeId::BOOL => "BOOL",
-
-        TypeId::CHAR => "\"CHAR\"",
-
-        TypeId::INT2 => "INT2",
-        TypeId::INT4 => "INT4",
-        TypeId::INT8 => "INT8",
-
-        TypeId::OID => "OID",
-
-        TypeId::FLOAT4 => "FLOAT4",
-        TypeId::FLOAT8 => "FLOAT8",
-
-        TypeId::NUMERIC => "NUMERIC",
-
-        TypeId::TEXT => "TEXT",
-        TypeId::VARCHAR => "VARCHAR",
-        TypeId::BPCHAR => "BPCHAR",
-        TypeId::UNKNOWN => "UNKNOWN",
-        TypeId::NAME => "NAME",
-
-        TypeId::DATE => "DATE",
-        TypeId::TIME => "TIME",
-        TypeId::TIMESTAMP => "TIMESTAMP",
-        TypeId::TIMESTAMPTZ => "TIMESTAMPTZ",
-
-        TypeId::BYTEA => "BYTEA",
-
-        TypeId::UUID => "UUID",
-
-        TypeId::CIDR => "CIDR",
-        TypeId::INET => "INET",
-
-        TypeId::ARRAY_BOOL => "BOOL[]",
-
-        TypeId::ARRAY_CHAR => "\"CHAR\"[]",
-
-        TypeId::ARRAY_INT2 => "INT2[]",
-        TypeId::ARRAY_INT4 => "INT4[]",
-        TypeId::ARRAY_INT8 => "INT8[]",
-
-        TypeId::ARRAY_OID => "OID[]",
-
-        TypeId::ARRAY_FLOAT4 => "FLOAT4[]",
-        TypeId::ARRAY_FLOAT8 => "FLOAT8[]",
-
-        TypeId::ARRAY_TEXT => "TEXT[]",
-        TypeId::ARRAY_VARCHAR => "VARCHAR[]",
-        TypeId::ARRAY_BPCHAR => "BPCHAR[]",
-        TypeId::ARRAY_NAME => "NAME[]",
-
-        TypeId::ARRAY_NUMERIC => "NUMERIC[]",
-
-        TypeId::ARRAY_DATE => "DATE[]",
-        TypeId::ARRAY_TIME => "TIME[]",
-        TypeId::ARRAY_TIMESTAMP => "TIMESTAMP[]",
-        TypeId::ARRAY_TIMESTAMPTZ => "TIMESTAMPTZ[]",
-
-        TypeId::ARRAY_BYTEA => "BYTEA[]",
-
-        TypeId::ARRAY_UUID => "UUID[]",
-
-        TypeId::ARRAY_CIDR => "CIDR[]",
-        TypeId::ARRAY_INET => "INET[]",
-
-        TypeId::JSON => "JSON",
-        TypeId::JSONB => "JSONB",
-
-        TypeId::RECORD => "RECORD",
-        TypeId::ARRAY_RECORD => "RECORD[]",
-
-        _ => {
-            return None;
-        }
-    })
+    false
 }
