@@ -23,6 +23,15 @@
 //! | `chrono::DateTime<Utc>`               | DATETIME                                             |
 //! | `chrono::DateTime<Local>`             | DATETIME                                             |
 //!
+//! ### [`uuid`](https://crates.io/crates/uuid)
+//!
+//! Requires the `uuid` Cargo feature flag.
+//!
+//! | Rust type                             | Sqlite type(s)                                       |
+//! |---------------------------------------|------------------------------------------------------|
+//! | `uuid::Uuid`                          | BLOB, TEXT                                           |
+//! | `uuid::adapter::Hyphenated`           | TEXT                                                 |
+//!
 //! # Nullable
 //!
 //! In addition, `Option<T>` is supported where `T` implements `Type`. An `Option<T>` represents
@@ -38,3 +47,5 @@ mod int;
 #[cfg(feature = "json")]
 mod json;
 mod str;
+#[cfg(feature = "uuid")]
+mod uuid;
